#include "delegate.h"

#include <QComboBox>
#include <QDoubleSpinBox>
#include <QStringListModel>

/*!
 * \class VariableEditDelegate
 * \brief Handles display of model data for VariableAdjustWidget.
 *
 * This class is heavily based on the example
 * [SpinBoxDelegate](http://doc.qt.io/qt-5/qtwidgets-itemviews-spinboxdelegate-example.html)
 * provided by the Qt framework.
 */

/*!
 * \brief Creates the editor based on index.
 *
 * \param parent The parent widget.
 * \param index The index of the current element.
 * \param option Editor option (unused).
 * \return the apropriate editor.
 */
QWidget *VariableEditDelegate::createEditor(QWidget *parent,
    const QStyleOptionViewItem &/* option */,
    const QModelIndex &index) const
{
    if (index.column() == 0 || index.column() == 1) {
    QDoubleSpinBox *editor = new QDoubleSpinBox(parent);
        editor->setFrame(false);
        editor->setSingleStep(0.1);
        editor->setDecimals(1);
        editor->setMaximum(30);
        editor->setMinimum(-30);
        editor->setAlignment(Qt::AlignRight);
        editor->setButtonSymbols(QAbstractSpinBox::PlusMinus);
        return editor;
    } else {
        QComboBox *editor = new QComboBox(parent);
        QStringList trig;
        trig << tr("cos") << tr("sin");
        QStringListModel *trigmodel = new QStringListModel(trig);
        editor->setModel(trigmodel);
        editor->setFrame(false);
        return editor;
    }

}

/*!
 * \brief Sets editor data from model. Triggered when the model is changed.
 *
 * \param editor The editor to provide with data.
 * \param index The index of the data.
 */
void VariableEditDelegate::setEditorData(QWidget *editor,
                                    const QModelIndex &index) const
{
    if (index.column() == 0 || index.column() == 1) {
        double value = index.model()->data(index, Qt::EditRole).toDouble();
        QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox*>(editor);
        spinBox->setValue(value);
    } else {
        int value = index.model()->data(index, Qt::EditRole).toInt();
        QComboBox *box = static_cast<QComboBox*>(editor);
        box->setCurrentIndex(value);
    }

}

/*!
 * \brief Sets model data from editor. Triggered when an editor is changed.
 *
 * \param editor The editor source.
 * \param model The model to edit.
 * \param index The index to edit.
 */
void VariableEditDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
    double value;
    if (index.column() == 0 || index.column() == 1) {
        QDoubleSpinBox *spinBox = static_cast<QDoubleSpinBox*>(editor);
        spinBox->interpretText();
        value = spinBox->value();
    } else {
        QComboBox *box = static_cast<QComboBox*>(editor);
        value = box->currentIndex();
    }
    model->setData(index, value, Qt::EditRole);
}

/*!
 * \brief Handles editor styling.
 *
 * \param editor The editor to style.
 * \param option Styling option.
 * \param index The data index (unused).
 */
void VariableEditDelegate::updateEditorGeometry(QWidget *editor,
    const QStyleOptionViewItem &option, const QModelIndex &/* index */) const
{
    editor->setGeometry(option.rect);
}
