#include "drawwidget.h"

#include <QMessageBox>
#include <QVBoxLayout>

#include <math.h>

/*!
 * \class DrawWidget
 * \brief The DrawWidget class plots a rosette based on the parameters found in its model.
 *
 * It inherits QWdiget and makes use of QCustomPlot.
 */

/*!
 * \brief Constructs the DrawWidget. Sets up \a customPlot, main \a graph and layout.
 *
 * \param parent The parent of the widget.
 */
DrawWidget::DrawWidget(QWidget *parent) : QWidget(parent)
{

    customPlot = new QCustomPlot();
    graph = new QCPCurve(customPlot->xAxis, customPlot->yAxis);

    customPlot->addPlottable(graph);
    customPlot->xAxis->setVisible(false);
    customPlot->yAxis->setVisible(false);
    customPlot->xAxis->grid()->setVisible(false);
    customPlot->yAxis->grid()->setVisible(false);
    customPlot->replot();

    QVBoxLayout *mainLayout = new QVBoxLayout;
        mainLayout->setSizeConstraint(QLayout::SetNoConstraint);
        mainLayout->addWidget(customPlot);
        setLayout(mainLayout);

}

/*!
 * \brief Destroys DrawWidget.
 *
 * Deletes \a customPlot, \a graph and \a model.
 */
DrawWidget::~DrawWidget()
{
    delete customPlot;
    delete graph;
    delete model;
}

/*!
 * \brief Sets a new model.
 *
 * \param newModel The new model to display.
 */
void DrawWidget::setModel(QStandardItemModel *newModel)
{
    this->model = newModel;
    connect(model, SIGNAL(itemChanged(QStandardItem*)),
            this, SLOT(itemChanged(QStandardItem*)));
    generatePlotData();
}

/*!
 * \brief Generates plot data from model to display with customPlot.
 */
void DrawWidget::generatePlotData()
{
    double interval = (2 * M_PI) / (5000.0);
    double tick = 0;

//    Reset the variables in preparation for new data
    graph->clearData();
    xdata.clear();
    ydata.clear();
    zdata.clear();

    for (int i = 0; i < 5000; ++i) {
        zdata.append(tick);
        double tempXdata = 0;
        double tempYdata = 0;

        // Calculate data points
        for (int row = 0; row < model->rowCount(); ++row) {
            double tempAmplitude = model->data(model->index(row,0)).toDouble();
            double tempFrequency = model->data(model->index(row,1)).toDouble();
            int tempxtrig = model->data(model->index(row,2)).toDouble();
            int tempytrig = model->data(model->index(row,3)).toDouble();

            // Test for sine/cosine components. 0 is cosine, 1 is sine.
            (tempxtrig == 0)
                    ? (tempXdata += tempAmplitude * cos(tempFrequency*tick))
                    : (tempXdata += tempAmplitude * sin(tempFrequency*tick));
            (tempytrig == 0)
                    ? (tempYdata += tempAmplitude * cos(tempFrequency*tick))
                    : (tempYdata += tempAmplitude * sin(tempFrequency*tick));
        }

        xdata.append(tempXdata);
        ydata.append(tempYdata);
        tick += interval;
    }
    graph->setData(zdata, xdata, ydata);

    // Rescale the axis for proper display
    graph->rescaleAxes();
    customPlot->xAxis->setRangeUpper(customPlot->xAxis->range().upper+1);
    customPlot->yAxis->setRangeUpper(customPlot->yAxis->range().upper+1);
    customPlot->xAxis->setRangeLower(customPlot->xAxis->range().lower-1);
    customPlot->yAxis->setRangeLower(customPlot->yAxis->range().lower-1);

    customPlot->replot();

}

/*!
 * \brief Is executed when an item has changed in the model.
 *
 * Triggers a recalculation of plot data with generatePlotData().
 * \param item The changed item, unused.
 */
void DrawWidget::itemChanged(QStandardItem *item)
{
    (void)item;
    generatePlotData();
}

/*!
 * \brief Is executed when a row is deleted.
 *
 * Triggers a recalculation of plot data with generatePlotData().
 */
void DrawWidget::rowDeleted()
{
    generatePlotData();
}

/*!
 * \brief Saves current plot as image.
 *
 * \param filename Name of the saved file.
 * \return \c True if the save is successfull; otherwise \c False.
 */
bool DrawWidget::saveImage(QString filename)
{
    customPlot->setAntialiasedElements(QCP::aeAll);
    bool ok = customPlot->savePng(filename+".png",
                                  customPlot->height()*2,
                                  customPlot->width()*2,
                                  2.0, -1);

    QMessageBox *reply = new QMessageBox();
    reply->setWindowTitle(tr("Save plot"));
    reply->setText(tr("Save parameters?"));
    reply->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    reply->setDefaultButton(QMessageBox::Yes);
    int ret = reply->exec();
    if (ret == QMessageBox::Yes) {
        ok = ok && saveParameters(filename);
    }

    return ok;
}

/*!
 * \brief Saves the current parameters to file.
 *
 * \param filename Name of the saved file.
 * \return \c True if the save is successfull; otherwise \c False.
 */
bool DrawWidget::saveParameters(QString filename)
{
    QFile outfile(filename);
    if(!outfile.open(QFile::WriteOnly |
                      QFile::Text))
        {
            return false;
        }

    QTextStream out(&outfile);
    QString temp;
    QTextStream stream(&temp);
    for (int row = 0; row < model->rowCount(); ++row) {
        for (int col = 0; col < model->columnCount()-2; ++col) {
            out << model->data(model->index(row,col)).toDouble() << ", ";
        }
        for (int col = model->columnCount()-2; col < model->columnCount(); ++col) {

            stream << model->data(model->index(row,col)).toDouble() << " ";
        }
    }
    out << temp;
    out << endl;

    outfile.flush();
    outfile.close();
    return true;
}
