#include "fourieranalysis.h"

#include "fourierdisplaywidget.h"

#include "math.h"
#include "opencv/cv.h"

#include <QFile>
#include <QListWidget>
#include <QStringList>

#define NFFT 2048
/*!
 * \class FourierAnalysis
 * \brief The FourierAnalysis class handles data adjustment and Fourier analysis of trace data.
 */

/*!
 * \brief Initiates analysis of a sampled and normalized trace.
 * \param trace New sample points.
 * \return a new model with the results of the Fourier analysis.
 */
QStandardItemModel* FourierAnalysis::analyze(QVector<QPointF> *trace)
{
    cv::Mat adjustedSamples = adjustSamplePoints(trace);

    QStandardItemModel *results = fourier(adjustedSamples);
    return results;
}

/*!
 * \brief Adjusts the sample points.
 * \param normalizedSamples The sample points.
 * \return adjusted sample points in a cv::Mat.
 */
cv::Mat FourierAnalysis::adjustSamplePoints(QVector<QPointF> *normalizedSamples)
{

    // Adjust relationship between x- and y-coordinates.
    int N = normalizedSamples->length();
    for(int i = 0; i < N; ++i) {
        (*normalizedSamples)[i].rx() *= 1.337;
    }

    // Calculate estimate of trace length
    QVector<double> *sampleDistOld = new QVector<double>(N, 0);
    for (int i = 1; i < N; ++i) {
        double xCurrent = normalizedSamples->at(i).x();
        double xLast = normalizedSamples->at(i-1).x();
        double yCurrent = normalizedSamples->at(i).y();
        double yLast = normalizedSamples->at(i-1).y();
        (*sampleDistOld)[i] = sqrt(pow(xCurrent-xLast, 2) + pow(yCurrent - yLast, 2))
                + sampleDistOld->at(i-1);
    }

    // Adjust sample points by mean value
    meanAdjust(normalizedSamples);

    cv::Mat_<float> adjustedSamples(2, NFFT, CV_32F);
    int j = 1;
    float step = sampleDistOld->last() / (NFFT-1);

    // Calulate new equidistant sample points based on the old sample points.
    for (int i = 0; i < NFFT; ++i) {

        if (i*step > sampleDistOld->at(j)) // Keep track of old sample points
            ++j;

        if (j >= N) {
            break;
        } else {
            if ((sampleDistOld->at(j) - sampleDistOld->at(j-1)) == 0) { // Which two sample points interpolation is done between.
                ++j;
                if (j>=N)
                    break;
            } else {
                float xOldCurrent = normalizedSamples->at(j).x();
                float xOldLast = normalizedSamples->at(j-1).x();
                float yOldCurrent = normalizedSamples->at(j).y();
                float yOldLast = normalizedSamples->at(j-1).y();
                float distNew = i*step;
                float distOldCurrent = sampleDistOld->at(j);
                float distOldLast = sampleDistOld->at(j-1);

                // Interpolation
                float xNew = (xOldCurrent - xOldLast) / (distOldCurrent - distOldLast)
                        * (distNew - distOldLast) + xOldLast;
                float yNew = (yOldCurrent - yOldLast) / (distOldCurrent - distOldLast)
                        * (distNew - distOldLast) + yOldLast;
                adjustedSamples.row(0).col(i) = xNew;
                adjustedSamples.row(1).col(i) = yNew;
            }
        }

    }
    delete sampleDistOld;
    delete normalizedSamples;
    return adjustedSamples;
}

/*!
 * \brief Adjusts samples by subtracting their mean from all points.
 * \param samples Sample points to be adjusted.
 */
void FourierAnalysis::meanAdjust(QVector<QPointF> *samples)
{
    double xmean = 0;
    double ymean = 0;
    int N = samples->length();
    for (int i = 0; i < N; ++i) {
        xmean += samples->at(i).x();
        ymean += samples->at(i).y();
    }
    xmean = xmean/N;
    ymean = ymean/N;

    for (int i = 0; i < N; ++i) {
        (*samples)[i].rx() -= xmean;
        (*samples)[i].ry() -= ymean;
    }
}

/*!
 * \brief Performs Fourier analysis on the sample points.
 * \param samples Adjusted sample points.
 * \return a model of the detected paramters.
 */
QStandardItemModel* FourierAnalysis::fourier(cv::Mat samples)
{
    using namespace cv;

    // Pad matrix.
    Mat padded;
    int m = getOptimalDFTSize( samples.rows );
    int n = getOptimalDFTSize( samples.cols );
    copyMakeBorder(samples, padded, 0, m - samples.rows, 0, n - samples.cols,
                   BORDER_CONSTANT, Scalar::all(0));

    Mat planes[] = {Mat_<float>(padded), Mat::zeros(padded.size(), CV_32F)};
    Mat complexI;
    merge(planes, 2, complexI);

    // DFT
    dft(complexI, complexI, DFT_ROWS+DFT_COMPLEX_OUTPUT);
    split(complexI, planes);                   // planes[0] = Re(DFT(I), planes[1] = Im(DFT(I))

    Size mSize = samples.size();
    Mat_<float> amplitudes(mSize, CV_32F);
    Mat_<float> angles(mSize, CV_32F);

    // Calulate amplitudes and phase angles.
    magnitude(planes[0],planes[1],amplitudes );
    amplitudes /= std::sqrt(NFFT);
    phase(planes[0], planes[1], angles);

    // Sort indexes from amplitudes based on amplitude values.
    Mat_<int> sortedIdx(15, mSize.height, CV_32S);
    sortIdx(amplitudes.colRange(0,15), sortedIdx, CV_SORT_EVERY_ROW + CV_SORT_DESCENDING);

    // Detect significant frequencies.
    QVector<int> significantFrequncies;
    for (int x = 0; x < sortedIdx.cols; ++x) {
        if (sortedIdx.at<int>(0,x) == 0)
            continue;
        int freq = sortedIdx.at<int>(0,x);
        if (sortedIdx.at<int>(0,x) != sortedIdx.at<int>(1,x)
                || (amplitudes.at<float>(0,freq) < 0.5
                && amplitudes.at<float>(1,freq) < 0.5)) {
                    break;
        }
        significantFrequncies << sortedIdx.at<int>(0,x);
    }

    // Find standard values,do sorting
    significantFrequncies = adjustValues(significantFrequncies, angles);
    //change displayresults to display from model
    QStandardItemModel *analysisModel = buildModel(amplitudes.colRange(0,15), significantFrequncies);
    FourierDisplayWidget::displayResults(amplitudes.colRange(0,15), significantFrequncies, angles);

    return analysisModel;
}

/*!
 * \brief FourierAnalysis::adjustValues
 */
QVector<int> FourierAnalysis::adjustValues(QVector<int> indexes, cv::Mat angles)
{
    for (int i = 0; i < indexes.length(); ++i) {
        int freq = indexes.at(i);
        float phix = angles.at<float>(0,freq) * 180/M_PI;
        float phiy = angles.at<float>(1,freq) * 180/M_PI;
        int qx = findQuadrant(phix);
        int qy = findQuadrant(phiy);
        if (qy-qx == -1 || qy-qx == 3) {// opposite direction == negative frequency
            indexes[i] = -indexes[i];
        }
    }
    return indexes;
}

int FourierAnalysis::findQuadrant(float angle)
{
    if (angle < 90.0)
        return 1;
    else if (angle < 180.0)
        return 2;
    else if (angle < 270.0)
        return 3;
    else
        return 4;
}

/*!
 * \brief Builds a model of the detected parameters.
 * \param values Amplitude values.
 * \param indexes The significant indexes.
 * \return a new model.
 */
QStandardItemModel* FourierAnalysis::buildModel(cv::Mat values, QVector<int> indexes)
{
    QList<QStandardItem* > amplitudes;
    QList<QStandardItem* > frequencies;
    QList<QStandardItem* > xtrig;
    QList<QStandardItem* > ytrig;

    for(int i = 0; i < indexes.length(); ++i){
        int tf = indexes.at(i);
        int idx = std::abs(tf);
        float ta = (values.at<float>(0,idx) + values.at<float>(1,idx))/2.0;
        amplitudes << new QStandardItem(QString::number(ta));
        frequencies << new QStandardItem(QString::number(tf));
        xtrig << new QStandardItem("0");
        ytrig << new QStandardItem("1");
    }

    QStandardItemModel *model = new QStandardItemModel();

    model->appendColumn(amplitudes);
    model->appendColumn(frequencies);
    model->appendColumn(xtrig);
    model->appendColumn(ytrig);

    QList<QString> header;
    header << QObject::tr("Ampl") << QObject::tr("Freq") << "X" << "Y";
    model->setHorizontalHeaderLabels(header);
    model->sort(1);
    return model;
}
