#include "fourierdisplaywidget.h"

#include <qcustomplot/qcustomplot.h>
#include <QLabel>
#include <QStringList>

#define NSAMPLES 1200
#define DELTA 1.0/NSAMPLES

/*!
 * \class FourierDisplayWidget
 * \brief The FourierDisplayWidget class displays results from FourierAnalysis.
 */

/*!
 * \brief Displays the results from FourierAnalysis.
 *
 * \param amplitudes Amplitudes from the Fourier analysis.
 * \param indexes Indexes of significant components.
 * \param angles Angles from the Fourier analysis.
 */
void FourierDisplayWidget::displayResults(cv::Mat amplitudes, QVector<int> indexes, cv::Mat angles)
{
    QWidget *w = new QWidget();
    w->setWindowTitle(tr("Fourier results"));

    QVector<double> *frequencies = new QVector<double>();
    QVector<double> *xamplitudes = new QVector<double>();
    QVector<double> *yamplitudes = new QVector<double>();

    // Prepare values for frequency/amplitude plots.
    for (int col = 0; col < amplitudes.cols; ++col){
        frequencies->append(col);
        xamplitudes->append(amplitudes.at<float>(0,col));
        yamplitudes->append(amplitudes.at<float>(1,col));
    }

    QVector<double> *graphValX = new QVector<double>();
    QVector<double> *graphValY = new QVector<double>();
    QVector<double> *steps = new QVector<double>();
    QStringList *phasevals = new QStringList();
    *phasevals << "Phase values: ";
    QString pi = QChar(0x03C0);
    for (int i = 0; i < NSAMPLES; ++i) {
        double xval = 0;
        double yval = 0;
        for (int c = 0; c < indexes.length(); ++c) {

            //X and Y plot values.
            int freq = indexes.at(c);
            float x = amplitudes.at<float>(0,abs(freq));
            float y = amplitudes.at<float>(1,abs(freq));
            float phix = angles.at<float>(0,abs(freq)) * 180/M_PI;
            float phiy = angles.at<float>(1,abs(freq)) * 180/M_PI;
            xval += x * cos(2*M_PI*(i* DELTA*freq) + phix);
            yval += y * cos(2*M_PI*(i* DELTA*freq) + phiy);

            if (i == 0)
                *phasevals << QString("Frequency %1: phase x = %2, phase y = %3").arg(freq)
                              .arg(phix).arg(phiy);
        }
        *graphValX << xval;
        *graphValY << yval;
        *steps << i;
    }

    //Frequency/amplitude plot.
    double minXAmp = *std::min_element(xamplitudes->begin(), xamplitudes->end());
    double maxXAmp = *std::max_element(xamplitudes->begin(), xamplitudes->end());
    double minYAmp = *std::min_element(yamplitudes->begin(), yamplitudes->end());
    double maxYAmp = *std::max_element(yamplitudes->begin(), yamplitudes->end());
    double minFreq = *std::min_element(frequencies->begin(), frequencies->end());
    double maxFreq = *std::max_element(frequencies->begin(), frequencies->end());
    QCustomPlot *xFreqAmpl = new QCustomPlot(w);
    xFreqAmpl->addGraph();
    xFreqAmpl->graph(0)->setData(*frequencies, *xamplitudes);
    xFreqAmpl->xAxis->setLabel("Frequencies");
    xFreqAmpl->yAxis->setLabel("Amplitudes");
    xFreqAmpl->xAxis->setAutoTickStep(false);
    xFreqAmpl->xAxis->setTickStep(1);
    xFreqAmpl->xAxis->setRange(minFreq, maxFreq);
    xFreqAmpl->yAxis->setRange(minXAmp, maxXAmp);
    QCustomPlot *yFreqAmpl = new QCustomPlot(w);
    yFreqAmpl->addGraph();
    yFreqAmpl->graph(0)->setData(*frequencies, *yamplitudes);
    yFreqAmpl->xAxis->setLabel("Frequencies");
    yFreqAmpl->yAxis->setLabel("Amplitudes");
    yFreqAmpl->xAxis->setAutoTickStep(false);
    yFreqAmpl->xAxis->setTickStep(1);
    yFreqAmpl->xAxis->setRange(minFreq, maxFreq);
    yFreqAmpl->yAxis->setRange(minYAmp, maxYAmp);

    // X and Y plots.
    double minVal = *std::min_element(graphValX->begin(), graphValX->end())-1;
    double maxVal = *std::max_element(graphValX->begin(), graphValX->end())+1;
    QVector<double> piTicks;
    QVector<QString> piLabels;
    piTicks << 0  << NSAMPLES/4.0 << NSAMPLES/2.0 << 3.0*NSAMPLES/4.0 << NSAMPLES;
    piLabels << "0" << "1/2"+pi << pi << "3/2"+pi << "2"+pi;
    QCustomPlot *xGraphPlot = new QCustomPlot(w);
    xGraphPlot->addGraph();
    xGraphPlot->graph(0)->setData(*steps, *graphValX);
    xGraphPlot->xAxis->setLabel("t");
    xGraphPlot->yAxis->setLabel("X = f(t)");
    xGraphPlot->xAxis->setRange(0, NSAMPLES);
    xGraphPlot->yAxis->setRange(minVal, maxVal);
    xGraphPlot->xAxis->setAutoTicks(false);
    xGraphPlot->xAxis->setAutoTickLabels(false);
    xGraphPlot->xAxis->setTickVector(piTicks);
    xGraphPlot->xAxis->setTickVectorLabels(piLabels);
    xGraphPlot->replot();
    QCustomPlot *yGraphPlot = new QCustomPlot(w);
    yGraphPlot->addGraph();
    yGraphPlot->graph(0)->setData(*steps, *graphValY);
    yGraphPlot->xAxis->setLabel("t");
    yGraphPlot->yAxis->setLabel("Y = f(t)");
    yGraphPlot->xAxis->setRange(0, NSAMPLES);
    yGraphPlot->yAxis->setRange(minVal, maxVal);
    yGraphPlot->xAxis->setAutoTicks(false);
    yGraphPlot->xAxis->setAutoTickLabels(false);
    yGraphPlot->xAxis->setTickVector(piTicks);
    yGraphPlot->xAxis->setTickVectorLabels(piLabels);
    yGraphPlot->replot();

    QLabel *phases = new QLabel(phasevals->join("\n"));

    // layout
    QGridLayout *layout = new QGridLayout(w);
    layout->addWidget(xFreqAmpl,0,0);
    layout->addWidget(yFreqAmpl,1,0);
    layout->addWidget(xGraphPlot,0,1);
    layout->addWidget(yGraphPlot,1,1);
    layout->addWidget(phases,2,0,1,2, Qt::AlignHCenter);
    layout->setColumnMinimumWidth(0, NSAMPLES/2);
    layout->setColumnMinimumWidth(1, NSAMPLES/2);
    layout->setRowMinimumHeight(0, NSAMPLES/4);
    layout->setRowMinimumHeight(1, NSAMPLES/4);
    w->setLayout(layout);
    w->show();
}
