#include "mainwindow.h"

#include <QDockWidget>
#include <QStandardItem>
#include <QTabWidget>
#include <QToolBar>

/*!
 * \class MainWindow
 * \brief The MainWindow class is the main window of RosAna.
 *
 * Setup of Gui, interactions and connections is done in this class.
 *
 * QTabWidget:
 * + ProcessWidget \a processImage.
 * + DrawWidget \a drawGraphWidget.
 *
 * QDockWidget:
 * + VariableAdjustWidget \a variableAdjustmentWidget.
 *
 * Dialogs:
 * + LoadExampleDialog \a loadExampleDialog.
 * + HelpDialogWidget \a helpWidget.
 *
 * Helpers:
 * + ReadExampleFile \a exampleValues.
 */

/*!
 * \brief Contructs MainWindow. Sets up Gui and interactions.
 * \param parent The parent widget.
 */
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    // Setup window specific options.
    this->setWindowTitle(tr("RosAna - The Rosette Analyzer"));
    this->addToolBar(tr("main toolbar"));
    this->statusBar()->show();

    exampleValues = new ReadExampleFile();

    // Initialize main widgets
    processImage = new ProcessWidget(this);
    drawGraphWidget = new DrawWidget(this);
    variableAdjustmentWidget = new VariableAdjustWidget(this);
    loadExampleDialog = new LoadExampleDialog(exampleValues->fetchNames(), this);
    helpWidget = new HelpDialogWidget(this);

    // Setup main window gui elements
    QTabWidget *tabWidget = new QTabWidget;
    tabWidget->addTab(processImage, tr("&Process"));
    tabWidget->addTab(drawGraphWidget, tr("&Synthesize"));
    this->setCentralWidget(tabWidget);

    QDockWidget *dockWidget = new QDockWidget();
       dockWidget->setAllowedAreas(Qt::RightDockWidgetArea);
       dockWidget->setWidget(variableAdjustmentWidget);
       this->addDockWidget(Qt::RightDockWidgetArea, dockWidget);
       QWidget* lTitleBar = dockWidget->titleBarWidget();
       QLabel* label = new QLabel(tr("Fourier components"));
       label->setContentsMargins(5,5,5,0);
       label->setAlignment(Qt::AlignHCenter);
       dockWidget->setTitleBarWidget(label);
       delete lTitleBar;

    // Connect signals and slots
    connect(variableAdjustmentWidget, SIGNAL(rowDeleted()),
            drawGraphWidget, SLOT(rowDeleted()));
    connect(loadExampleDialog, SIGNAL(selectedExample(int)),
            this, SLOT(setModels(int)));
    connect(processImage, SIGNAL(manualTraceInProgress(QString)),
            this, SLOT(setStatusBar(QString)));
    connect(processImage, SIGNAL(traceDone(QStandardItemModel*)),
            this, SLOT(tracedModel(QStandardItemModel*)));
    connect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(updateActionBar(int)));

    // Final setup of actions, shortcuts and widget initialization.
    setupToolbar();
    setShortcuts();
    setModels(0);
    tabWidget->setCurrentIndex(0);
}

/*!
 * \brief Gets the selected model from \a exampleValues and sets the new model for \a drawGraphWidget
 *  and \a variableAdjustmentWidget.
 * \param index The selected index
 */
void MainWindow::setModels(int index)
{
    QTabBar *t = this->findChildren<QTabBar *>(QString()).first();
    t->setCurrentIndex(1);
    drawGraphWidget->setModel(exampleValues->getModel(index));
    variableAdjustmentWidget->setModel(exampleValues->getModel(index));
    this->repaint();
}

/*!
 * \brief Destroys MainWindow. Deletes exampleValues.
 */
MainWindow::~MainWindow()
{
    delete exampleValues;
}

/*!
 * \brief Sets up the main toolbar, adds actions and shortcuts.
 */
void MainWindow::setupToolbar()
{
    QToolBar *toolbar = this->findChildren<QToolBar *>(QString()).first();

    QIcon::setThemeName("ThinkGreen");
    QAction *openImageAction = toolbar->addAction(QIcon::fromTheme("photo"),
                                                  tr("&Open image"),
                                                  this, SLOT(openImage()));
    QAction *saveImageAction = toolbar->addAction(QIcon::fromTheme("save"),
                                                 tr("&Save image"),
                                                 this, SLOT(saveImage()));
    QAction *startTraceAction = toolbar->addAction(QIcon::fromTheme("trace"),
                                                  tr("Start manual &trace"),
                                                  processImage, SLOT(manualTrace()));
    QAction *loadExampleAction = toolbar->addAction(QIcon::fromTheme("favorite"),
                                                   tr("&Load example rosette"),
                                                   this, SLOT(loadDialog()));
    QAction *helpAction = toolbar->addAction(QIcon::fromTheme("help"),
                                                tr("Help"),
                                                this, SLOT(helpDialog()));
    startTraceAction->setShortcut(Qt::CTRL + Qt::Key_T);
    loadExampleAction->setShortcut(Qt::CTRL + Qt::Key_L);
    saveImageAction->setShortcut(QKeySequence::Save);
    openImageAction->setShortcut(QKeySequence::Open);
    helpAction->setShortcut(QKeySequence::HelpContents);
}

/*!
 * \brief Adds extra shortcuts to MainWindow.
 */
void MainWindow::setShortcuts()
{
    QAction *closeWindow = new QAction(this);
    closeWindow->setShortcut(QKeySequence::Close);
    connect(closeWindow, SIGNAL(triggered()), this, SLOT(close()));
    this->addAction(closeWindow);

    QAction *maxWindow = new QAction(this);
    maxWindow->setShortcut(Qt::CTRL + Qt::Key_M);
    connect(maxWindow, SIGNAL(triggered()), this, SLOT(showMaximized()));
    this->addAction(maxWindow);

    QAction *minWindow = new QAction(this);
    minWindow->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_M);
    connect(minWindow, SIGNAL(triggered()), this, SLOT(showNormal()));
    this->addAction(minWindow);
}

/*!
 * \brief Executes LoadExampleDialog.
 */
void MainWindow::loadDialog(){
    loadExampleDialog->setWindowTitle(tr("Example rosettes"));
    loadExampleDialog->setModal(true);
    loadExampleDialog->exec();
    QTabBar *t = this->findChildren<QTabBar *>(QString()).first();
    t->setCurrentIndex(1);
}

/*!
 * \brief Executes HelpDialogWidget.
 */
void MainWindow::helpDialog()
{
    helpWidget->show();
    QTabBar *h = helpWidget->findChildren<QTabBar *>(QString()).first();
    h->setCurrentIndex(0);
    h->setFocus();
}

/*!
 * \brief Is executed when saveImageAction is executed. Opens a save file dialog.
 *
 * Detects the current mode and forwards the save trigger to either ProcessWidget or DrawWidget
 */
void MainWindow::saveImage()
{
    int tab = dynamic_cast<QTabWidget *>(this->centralWidget())->currentIndex();
    QString title;
    QString filetypes;
    if (tab == 0) {
        title = tr("Save trace");
        filetypes = tr("Text files (*.txt)");
    } else {
        title = tr("Save plot");
        filetypes = tr("Images (*.png *.xpm *.jpg)");
    }

    QString filename = QFileDialog::getSaveFileName(this, title, QDir::homePath(),
                                                    filetypes);

    bool saved;
    if(!filename.isEmpty()) {
        if(tab == 0) {
            saved = processImage->saveTrace(filename);
        } else {
            saved = drawGraphWidget->saveImage(filename);
        }
    } else {
        return;
    }

    if(saved) {
        QString message = tr("Image/plot saved to ") + filename;
        this->statusBar()->showMessage(message, 5000);
    } else {
        QMessageBox *error = new QMessageBox(QMessageBox::Critical, tr("Error"),
                                             tr("Something went wrong, image not saved"),
                                             QMessageBox::Ok, this);
        error->exec();
    }
}

/*!
 * \brief Is executed when openImageAction is triggered. Opens an open file dialog.
 */
void MainWindow::openImage()
{
    QString filename = QFileDialog::getOpenFileName(this, "Select image", QDir::homePath(),
                                                    tr("Images (*.png *.xpm *.jpg *.PNG *.XPM *.JPG)"));
    if(!filename.isEmpty()) {
        bool success = processImage->openImage(filename);
        if(!success) {
            QMessageBox *error = new QMessageBox(QMessageBox::Critical, tr("Error"),
                                                 tr("Could not load image"),
                                                 QMessageBox::Ok, this);
            error->exec();
        }
    }
    QTabBar *t = this->findChildren<QTabBar *>(QString()).first();
    t->setCurrentIndex(0);
}

/*!
 * \brief Is executed when a new model has been generated from Fourier analysis.
 *
 * Sets the new model for DrawWidget and VariableAdjustWidget. Sets the current mode to synthesize.
 * \param model The model resulting from Fourier analysis.
 */
void MainWindow::tracedModel(QStandardItemModel *model)
{
    QTabBar *t = this->findChildren<QTabBar *>(QString()).first();
    t->setCurrentIndex(1);
    drawGraphWidget->setModel(model);
    variableAdjustmentWidget->setModel(model);
    this->repaint();
    this->clearStatusBar();
}

/*!
 * \brief Is executed when a signal is triggered with a message for the status bar.
 * \param text The text to display.
 */
void MainWindow::setStatusBar(QString text)
{
    this->statusBar()->showMessage(text);
}

/*!
 * \brief Is executed when a signal is triggered to clear the status bar.
 */
void MainWindow::clearStatusBar()
{
    this->statusBar()->clearMessage();
}

/*!
 * \brief Updates the action bar icons. Is executed when the active tab is changed.
 * \param index The current tab index.
 */
void MainWindow::updateActionBar(int index)
{
    QToolBar *t = this->findChildren<QToolBar *>(QString()).first();
    if (index == 0) {
        t->actions().at(2)->setEnabled(true);
    } else {
        t->actions().at(2)->setEnabled(false);
    }
}
