#include "processwidget.h"

#include "fourieranalysis.h"

#include <QMouseEvent>
#include <QTextStream>
#include <QVBoxLayout>

using namespace cv;
/*!
 * \class ProcessWidget
 * \brief The ProcessWidget class handles image display and pattern tracing.
 */

/*!
 * \brief Constructs ProcessWidget.
 *
 * Reads an example file and sets up gui elements.
 * \param parent The parent widget.
 */
ProcessWidget::ProcessWidget(QWidget *parent) : QWidget(parent)
{
    QFile file(":/turk.jpg");
    // OpenCV image from qt resource.
    if(file.open(QIODevice::ReadOnly))
        {
            qint64 sz = file.size();
            std::vector<uchar> buf(sz);
            file.read((char*)buf.data(), sz);
            img = imdecode(buf,IMREAD_COLOR);
        }
    imwidget = new CVImageWidget(this);
    imwidget->showImage(img);

    manualTraceActive = false;

    QVBoxLayout *mainLayout = new QVBoxLayout;
        mainLayout->setSizeConstraint(QLayout::SetNoConstraint);
        mainLayout->addWidget(imwidget);
        setLayout(mainLayout);
}

/*!
 * \brief Destroys ProcessWidget.
 *
 * Releases resources held by \a img and \a traceImg.
 */
ProcessWidget::~ProcessWidget()
{
    img.release();
    traceImg.release();
}

/*!
 * \brief Loads an image from disk.
 * \param filepath The image file path.
 * \return \c True if the load is successful; otherwise \c False.
 */
bool ProcessWidget::openImage(const QString &filepath)
{
    img = imread(filepath.toStdString());
    if (! img.data)
        return false;
    imwidget->showImage(img);
    this->repaint();
    return true;
}

/*!
 * \brief Saves a normalized pattern trace to disk.
 *
 * Format: x1, y1, \\n x2, y2, \\n etc.
 * \param filepath The image file path.
 * \return \c True if the save is successful; otherwise \c False.
 */
bool ProcessWidget::saveTrace(QString filepath)
{
    //write trace to file, x, y \n
    QFile outfile(filepath);
    if(!outfile.open(QFile::WriteOnly |
                      QFile::Text))
        {
            return false;
        }
    QVector<QPointF> *normalizedSamples = normalize(&trace);
    QTextStream out(&outfile);

    while (!normalizedSamples->empty()) {
        QPointF p = normalizedSamples->takeFirst();
        out << p.x() << ", " << p.y() << endl;
    }

    outfile.flush();
    outfile.close();
    return true;
}

/*!
 * \brief Initiates manual trace
 */
void ProcessWidget::manualTrace()
{
    emit manualTraceInProgress(tr("Manual trace in progress. Press ENTER when finished"));
    this->setFocus();
    this->setFocusPolicy(Qt::ClickFocus);
    traceImg = img.clone();
    imwidget->showImage(img);
    trace.clear();
    manualTraceActive = true;
}

/*!
 * \brief Records mouse clicks if a trace is active.
 * \param ev the mouse event.
 */
void ProcessWidget::mousePressEvent(QMouseEvent *ev)
{
    if(manualTraceActive) {
        QPoint p = ev->pos() - imwidget->pos();
        trace << p;
        Point start = convertPoint(p);
        Point end;
        if(trace.length() == 1) {
            end = start;
        } else {
            QPoint t = trace.at(trace.length()-2);
            end = convertPoint(t);
        }
        drawLine(start, end);
    }
}

/*!
 * \brief Captures key press events.
 *
 * Using the Enter key triggers a stop to the current trace. The trace is normalized and passed
 * on to FourierAnalysis::analyze(QVector<QPointF>)
 *
 * \param ev Key press event.
 */
void ProcessWidget::keyPressEvent(QKeyEvent *ev)
{
    if(manualTraceActive && ev->key()==Qt::Key_Return) {
        QVector<QPointF> *normalizedSamples = normalize(&trace);
        QStandardItemModel *resultModel = FourierAnalysis::analyze(normalizedSamples);
        emit traceDone(resultModel);
        manualTraceActive = false;
        this->repaint();
    }
}

/*!
 * \brief Converts between Qt::Qpoint and cv::Point.
 * \param old QPoint to be converted.
 * \return a cv::Point.
 */
Point ProcessWidget::convertPoint(QPoint old)
{
    return Point(old.x(), old.y());
}

/*!
 * \brief Draws a line on \a traceImg.
 * \param start Start point.
 * \param end End point.
 */
void ProcessWidget::drawLine(Point start, Point end )
{
  int thickness = 2;
  int lineType = 8;
  line( traceImg,
        start,
        end,
        Scalar( 0, 0, 0 ),
        thickness,
        lineType );
  imwidget->showImage(traceImg);
  this->repaint();
}

/*!
 * \brief Normalizes trace points.
 * \param samples The trace to be normalized.
 * \return a normalized trace on the range [0,1].
 */
QVector<QPointF>* ProcessWidget::normalize(QVector<QPoint> *samples)
{
    QVector<QPointF> *normalized = new QVector<QPointF>(samples->length());
    double xfactor = img.size().width;
    double yfactor = img.size().height;

    for (int i = 0; i < samples->length(); ++i) {
        QPoint p = samples->at(i);
        (*normalized)[i] = QPointF(p.x()/xfactor, p.y()/yfactor);
    }
    return normalized;
}

/*!
 * \fn void ProcessWidget::manualTraceInProgress(QString text)
 * \brief This signal is emitted once a trace has been initiated.
 * \param text The message
 */
/*!
 * \fn void ProcessWidget::traceDone(QStandardItemModel *traceModel)
 * \brief This signal is emitted when the trace is completed.
 * \param traceModel The model resulting from the Fourier Analysis.
 */
