#include "readexamplefile.h"

#include <QFile>
#include <QStandardItem>
#include <QTextStream>

/*!
 * \class ReadExampleFile
 * \brief The ReadExampleFile class reads example rosettes from file.
 *
 * Stores the values in a model such that other classes can get the model
 */

/*!
 * \brief Constructs an instance of ReadExampleFile and calls readFile().
 */
ReadExampleFile::ReadExampleFile()
{
    values = new QVector<QStringList>;
    readFile();
}

/*!
 * \brief Destroys ReadExampleFile and deleted \a values.
 */
ReadExampleFile::~ReadExampleFile()
{
    delete values;
}

/*!
 * \brief reads the file of example rosettes.
 *
 * The examples are on the format "name, a1, f1, a2, f2, ....
 * Values are stored in \a nameList and \a values.
 */
void ReadExampleFile::readFile()
{
    QFile file(":/exampleValues.txt");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return;

    QTextStream stream(&file);
    while (!stream.atEnd()) {
        QStringList line = stream.readLine().split(", ", QString::SkipEmptyParts);
        nameList << line.takeFirst();
        values->append(line);
    }
    file.close();
}

/*!
 * \brief returns the names of example rosettes.
 * \return a list of example rosette names.
 */
QStringList ReadExampleFile::fetchNames() {
    return nameList;
}

/*!
 * \brief Returns a model of an example rosette.
 * \param index of which rosette example to load.
 * \return a new model containing the parameters of the example rosette.
 */
QStandardItemModel *ReadExampleFile::getModel(int index)
{
    QStringList data = values->at(index);

    QStandardItemModel *newModel = new QStandardItemModel();

    QList<QStandardItem* > amplitudes;
    QList<QStandardItem* > frequencies;
    QList<QStandardItem* > xtrig;
    QList<QStandardItem* > ytrig;

    for(int i = 0; i < data.length(); i++){
        QStandardItem *tempdata = new QStandardItem(data.at(i));

        if (i%2 == 0) {
            amplitudes << tempdata;
        } else {
            frequencies << tempdata;
        }

        if (i < data.length()/2) {
            xtrig << new QStandardItem("0");
        } else {
            ytrig << new QStandardItem("1");
        }
    }

    newModel->appendColumn(amplitudes);
    newModel->appendColumn(frequencies);
    newModel->appendColumn(xtrig);
    newModel->appendColumn(ytrig);

    QStringList header;
    header << QObject::tr("Ampl") << QObject::tr("Freq") << "X" << "Y";
    newModel->setHorizontalHeaderLabels(header);
    return newModel;
}
