#include "variableadjustwidget.h"

#include <QGridLayout>
#include <QHeaderView>
#include <QModelIndex>
#include <QModelIndexList>

/*!
 * \class VariableAdjustWidget
 * \brief The VariableAdjustWidget class holds the GUI for rosette parameters.
 *
 * The VariableAdjustWidget class inherits QWidget.
 * Parameters are set from a model shared with DrawWidget.
 */

/*!
 * \brief Constructs VariableAdjustWidget.
 *
 * Initialize all GUI elements:
 *  + QTableView parametersTableView
 *  + VariableEditDelegate viewDelegate
 *  + QDataWidgetMapper mapper
 *  + QPushButton addRowButton
 *  + QPushButton deleteRowButton
 * Connect signals and set layout
 *
 * \param parent The parent of VariableAdjustWidget.
 */
VariableAdjustWidget::VariableAdjustWidget(QWidget *parent) : QWidget(parent)
{

    parametersTableView = new QTableView(this);
    viewDelegate = new VariableEditDelegate();
    parametersTableView->setItemDelegate(viewDelegate);
    parametersTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    parametersTableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

    mapper = new QDataWidgetMapper(this);

    addRowButton = new QPushButton(this);
    deleteRowButton = new QPushButton(this);
    addRowButton->setText(tr("Add row"));
    deleteRowButton->setText(tr("Delete row"));

    connect(addRowButton, SIGNAL(clicked()), this, SLOT(addRow()));
    connect(deleteRowButton, SIGNAL(clicked()), this, SLOT(deleteRow()));

    QGridLayout *layout = new QGridLayout(this);
    layout->addWidget(parametersTableView, 0, 0, 1, 2);
    layout->addWidget(addRowButton, 1, 0);
    layout->addWidget(deleteRowButton, 1, 1);
    setLayout(layout);

}

/*!
 * \brief Destroys VariableAdjustWidget, deletes \a viewDelegate.
 */
VariableAdjustWidget::~VariableAdjustWidget()
{
    delete viewDelegate;
}

/*!
 * \brief Sets a new model for \a parametersTableView and \a mapper.
 * \param newModel The new model to display
 */
void VariableAdjustWidget::setModel(QStandardItemModel *newModel)
{
    parametersTableView->setModel(newModel);
    mapper->setModel(newModel);
    mapper->setItemDelegate(viewDelegate);
    mapper->addMapping(parametersTableView, 0);
    mapper->addMapping(parametersTableView, 1);
    mapper->addMapping(parametersTableView, 2, "currentIndex");

    for (int row = 0; row < newModel->rowCount(); ++row) {
        for (int col = 0; col < newModel->columnCount(); ++col) {
            parametersTableView->openPersistentEditor(newModel->index(row, col, QModelIndex()));
        }
    }

}

/*!
 * \fn VariableAdjustWidget::rowDeleted()
 * This signal is emitted when a row has been deleted from the model.
 */

/*!
 * \brief Is executed when \a addRowButton is clicked.
 * Adds a new row to the internal model.
 */
void VariableAdjustWidget::addRow()
{
    QStandardItemModel *model = { static_cast<QStandardItemModel *>(parametersTableView->model()) };
    QList<QStandardItem* > newRow;
    for (int i = 0; i < model->columnCount()-1; ++i)
        newRow << new QStandardItem("0");
    newRow << new QStandardItem("1");
    model->appendRow(newRow);

    for (int col = 0; col < model->columnCount(); ++col) {
        parametersTableView->openPersistentEditor(
                    model->index(model->rowCount()-1, col, QModelIndex()));
    }
    repaint();
}

/*!
 * \brief Is executed when \a deleteRowButton is clicked.
 * Gets the selected \a index from \a parametersTableView and deletes the corresponding row in its
 * internal model. Emits signal \a rowDeleted()
 */
void VariableAdjustWidget::deleteRow()
{
    QModelIndexList selectedIndexes = parametersTableView->selectionModel()->selection().indexes();
    if (!selectedIndexes.isEmpty()) {
        int index = selectedIndexes.first().row();
        QStandardItemModel *model = {static_cast<QStandardItemModel *>(parametersTableView->model())};
        model->removeRow(index);
        emit rowDeleted();
    }
}
